@@ -31,7 +31,7 @@
  * How long do we hold an HTTP client connection if
  * we are awaiting payment before giving up?
  */
-#define CHECK_PAYMENT_TIMEOUT GNUNET_TIME_relative_multiply ( \
+#define CHECK_PAYMENT_GENERIC_TIMEOUT GNUNET_TIME_relative_multiply ( \
     GNUNET_TIME_UNIT_MINUTES, 30)
 
 
@@ -100,7 +100,7 @@ struct BackupContext
   /**
    * Used while we are awaiting proposal creation.
    */
-  struct TALER_MERCHANT_PostOrdersOperation *po;
+  struct TALER_MERCHANT_PostOrdersHandle *po;
 
   /**
    * Used while we are waiting payment.
@@ -109,6 +109,7 @@ struct BackupContext
 
   /**
    * HTTP response code to use on resume, if non-NULL.
+
    */
   struct MHD_Response *resp;
 
@@ -305,35 +306,30 @@ make_payment_request (const char *order_id,
  * /contract request to a merchant.
  *
  * @param cls our `struct BackupContext`
- * @param hr HTTP response details
- * @param order_id order id of the newly created order
- * @param token the claim token generated by the merchant (NULL if
- *        it wasn't generated).
+ * @param por response details
  */
 static void
 proposal_cb (void *cls,
-             const struct TALER_MERCHANT_HttpResponse *hr,
-             const char *order_id,
-             const struct TALER_ClaimTokenP *token)
+             const struct TALER_MERCHANT_PostOrdersReply *por)
 {
   struct BackupContext *bc = cls;
   enum SYNC_DB_QueryStatus qs;
 
   bc->po = NULL;
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Resuming connection with order `%s'\n",
-              order_id);
   GNUNET_CONTAINER_DLL_remove (bc_head,
                                bc_tail,
                                bc);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Resuming connection with order `%s'\n",
+              bc->order_id);
   MHD_resume_connection (bc->con);
   SH_trigger_daemon ();
-  if (MHD_HTTP_OK != hr->http_status)
+  if (MHD_HTTP_OK != por->hr.http_status)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Backend returned status %u/%u\n",
-                hr->http_status,
-                (unsigned int) hr->ec);
+                por->hr.http_status,
+                (unsigned int) por->hr.ec);
     GNUNET_break (0);
     bc->resp = TALER_MHD_make_json_pack ("{s:I, s:s, s:I, s:I, s:O}",
                                          "code",
@@ -342,27 +338,27 @@ proposal_cb (void *cls,
                                          "hint",
                                          "Failed to setup order with merchant backend",
                                          "backend-ec",
-                                         (json_int_t) hr->ec,
+                                         (json_int_t) por->hr.ec,
                                          "backend-http-status",
-                                         (json_int_t) hr->http_status,
+                                         (json_int_t) por->hr.http_status,
                                          "backend-reply",
-                                         hr->reply);
+                                         por->hr.reply);
     GNUNET_assert (NULL != bc->resp);
     bc->response_code = MHD_HTTP_FAILED_DEPENDENCY;
     return;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Storing payment request for order `%s'\n",
-              order_id);
+              por->details.ok.order_id);
   qs = db->store_payment_TR (db->cls,
                              &bc->account,
-                             order_id,
-                             token,
+                             por->details.ok.order_id,
+                             por->details.ok.token,
                              &SH_annual_fee);
   if (0 >= qs)
   {
     GNUNET_break (0);
-    bc->resp = TALER_MHD_make_error (TALER_EC_SYNC_PAYMENT_CREATE_DB_ERROR,
+    bc->resp = TALER_MHD_make_error (TALER_EC_GENERIC_DB_STORE_FAILED,
                                      "Failed to persist payment request in sync database");
     GNUNET_assert (NULL != bc->resp);
     bc->response_code = MHD_HTTP_INTERNAL_SERVER_ERROR;
@@ -370,9 +366,9 @@ proposal_cb (void *cls,
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Obtained fresh order `%s'\n",
-              order_id);
-  bc->resp = make_payment_request (order_id,
-                                   token);
+              por->details.ok.order_id);
+  bc->resp = make_payment_request (por->details.ok.order_id,
+                                   por->details.ok.token);
   GNUNET_assert (NULL != bc->resp);
   bc->response_code = MHD_HTTP_PAYMENT_REQUIRED;
 }
@@ -450,8 +446,8 @@ check_payment_cb (void *cls,
       if (0 <= qs)
         return; /* continue as planned */
       GNUNET_break (0);
-      bc->resp = TALER_MHD_make_error (TALER_EC_SYNC_PAYMENT_CONFIRM_DB_ERROR,
-                                       "Failed to persist payment confirmation in sync database");
+      bc->resp = TALER_MHD_make_error (TALER_EC_GENERIC_DB_STORE_FAILED,
+                                       "increment lifetime");
       GNUNET_assert (NULL != bc->resp);
       bc->response_code = MHD_HTTP_INTERNAL_SERVER_ERROR;
       return; /* continue as planned */
@@ -475,7 +471,7 @@ check_payment_cb (void *cls,
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Timeout waiting for payment\n");
-  bc->resp = TALER_MHD_make_error (TALER_EC_SYNC_PAYMENT_TIMEOUT,
+  bc->resp = TALER_MHD_make_error (TALER_EC_SYNC_PAYMENT_GENERIC_TIMEOUT,
                                    "Timeout awaiting promised payment");
   GNUNET_assert (NULL != bc->resp);
   bc->response_code = MHD_HTTP_REQUEST_TIMEOUT;
@@ -539,8 +535,8 @@ begin_payment (struct BackupContext *bc,
     struct MHD_Response *resp;
     MHD_RESULT ret;
 
-    resp = TALER_MHD_make_error (TALER_EC_SYNC_PAYMENT_CHECK_ORDER_DB_ERROR,
-                                 "Failed to check for existing orders in sync database");
+    resp = TALER_MHD_make_error (TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                 "pending payments");
     ret = MHD_queue_response (bc->con,
                               MHD_HTTP_INTERNAL_SERVER_ERROR,
                               resp);
@@ -633,16 +629,21 @@ handle_database_error (struct BackupContext *bc,
                   "Payment required, awaiting completion of `%s'\n",
                   order_id);
       await_payment (bc,
-                     CHECK_PAYMENT_TIMEOUT,
+                     CHECK_PAYMENT_GENERIC_TIMEOUT,
                      order_id);
     }
     return MHD_YES;
   case SYNC_DB_HARD_ERROR:
+    GNUNET_break (0);
+    return TALER_MHD_reply_with_error (bc->con,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_GENERIC_DB_COMMIT_FAILED,
+                                       NULL);
   case SYNC_DB_SOFT_ERROR:
     GNUNET_break (0);
     return TALER_MHD_reply_with_error (bc->con,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                       TALER_EC_SYNC_DATABASE_FETCH_ERROR,
+                                       TALER_EC_GENERIC_DB_SOFT_FAILURE,
                                        NULL);
   case SYNC_DB_NO_RESULTS:
     GNUNET_assert (0);
